/*
 *  Copyright 2001-2010 Mikhail Kryshen <mikhail@kryshen.net>
 *
 *  This file is part of Graph Game.
 *
 *  Graph Game is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Graph Game is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Graph Game.  If not, see <http://www.gnu.org/licenses/>.
 */
package kryshen.graphg;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.util.StringTokenizer;

public class RenderingManager {

    /* === double buffering === */

    Image buffer = null;
    Graphics offscreen = null;

    public RenderingManager() {
    }

    public void updateBuffer(Component c, Graphics g, Dimension size) {
       	if (buffer == null || 
	    buffer.getWidth(null) != size.width || 
	    buffer.getHeight(null) != size.height) {
	    
	    if (offscreen != null) offscreen.dispose();

	    buffer = c.createImage(size.width, size.height);
	    offscreen = buffer.getGraphics();

	    setAA(offscreen, AA_DEFAULT);
	}
	
	offscreen.setClip(g.getClip());
    }

    /* === anti-aliasing === */

    private static boolean aa = true;

    public static final int AA_OFF = 0;
    public static final int AA_TEXT = 1;
    public static final int AA_FULL = 2;
    public static final int AA_DEFAULT = AA_TEXT;

    static void setAA(Graphics g, final int aamode) {
        if (aa) {
            try {
                (new Object() {
                    void setAA(Graphics g) throws Throwable {
                        switch (aamode) {
                            case AA_TEXT:
                                ((Graphics2D) g).addRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF));
                                ((Graphics2D) g).addRenderingHints(new RenderingHints(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON));
                                break;
                            case AA_FULL:
                                ((Graphics2D) g).addRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
                                break;
                            default:
                                ((Graphics2D) g).addRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF));
                        }
                    }
                }).setAA(g);
            } catch (Throwable e) {
                System.err.println(Game.TITLE + ": " + e);
                System.err.println("    Antialiasing is not supported by your Java VM.");
                aa = false;
            }
        }
    }

    /* === drawing === */

    public static final int ALIGN_LEFT = 0;
    public static final int ALIGN_CENTER = 1;
    public static final int ALIGN_RIGHT = 2;

    public static void drawStringInRect(String s, int x, int y, 
					int w, int h, 
					int align, Graphics g) {

	FontMetrics fm = g.getFontMetrics();
	StringTokenizer st = new StringTokenizer(s, "\n");

	int sh = fm.getHeight();
	int sy = y + Math.round((h - sh * st.countTokens()) / 2F) + fm.getAscent();

	while (st.hasMoreTokens()) {
	    String line = st.nextToken();
	    int sx;
	    int sw = fm.stringWidth(line);

	    switch (align) {
	    case ALIGN_LEFT:
		sx = x;
	    	break;
	    case ALIGN_RIGHT:
		sx = x + w - sw;
		break;
	    default:  // center
		sx = x + (w - sw) / 2;
	    }

	    g.drawString(line, sx, sy);
	    sy += sh;
	}
    }

    public static void drawStringBorder(String s, int x, int y, 				 
					int w, int h, int align, 
					int borderWidth, Graphics g) {

	FontMetrics fm = g.getFontMetrics();
	StringTokenizer st = new StringTokenizer(s, "\n");
	
	int sh = fm.getHeight();

	int bx1 = Integer.MAX_VALUE;
	int bx2 = Integer.MIN_VALUE;

	int bh = sh * st.countTokens();

	int sy = y + Math.round((h - bh) / 2F);
	
	while (st.hasMoreTokens()) {
	    String line = st.nextToken();
	    int sx;
	    int sw = fm.stringWidth(line);
	    
	    switch (align) {
	    case ALIGN_LEFT:
		sx = x;
		break;
	    case ALIGN_RIGHT:
		sx = x + w - sw;
		break;
	    default:  // center
		sx = x + (w - sw) / 2;
	    }

	    if (sx < bx1) bx1 = sx;
	    if (sx + sw > bx2) bx2 = sx + sw;
	}

	g.fillRect(bx1 - borderWidth, sy - borderWidth, 
		   bx2 - bx1 + borderWidth * 2,
		   bh + borderWidth * 2);
    }

    public static void drawStringInRect(String s, int x, int y, 				 
					int w, int h, int align, 
					int shadowOffset, 
					Color shadowColor, Graphics g) {
	if (shadowOffset > 0) {
	    Color old = g.getColor();
	    g.setColor(shadowColor);
	    drawStringInRect(s, x + shadowOffset, y + shadowOffset, 
			     w, h, align, g);
	    g.setColor(old);
	}

	drawStringInRect(s, x, y, w, h, align, g);
    }
}
